@@ -2,10 +2,11 @@
 This module contains Edge and Arrow subclasses for this behavioral package.
 """
 
+from typing import Literal
 from .....logger import logger
 from ...config import WeightSetting
 from ...colors import Color
-from ...graph import Arrow, Edge, Vertex
+from ...graph import Arrow, DataEdge, Edge, Vertex
 from ..filesystem.entities import File, Handle
 from ..network.entities import Connection, Socket
 from .entities import Data, Diff
@@ -63,7 +64,7 @@ class IsDiffOf(Edge):
 
 
 
-class IsReadBy(Arrow):
+class IsReadBy(Arrow, IsDiffOf):
 
     """
     This kind of arrow indicates that a Diff node only read from a vector.
@@ -76,7 +77,7 @@ class IsReadBy(Arrow):
 
 
 
-class WritesInto(Arrow):
+class WritesInto(Arrow, IsDiffOf):
 
     """
     This kind of arrow indicates that a Diff node only wrote to a vector.
@@ -89,7 +90,7 @@ class WritesInto(Arrow):
 
 
 
-class HasSimilarContent(Edge):
+class HasSimilarContent(DataEdge):
 
     """
     This kind of arrow indicates that two Diff nodes have buffers with a certain similarity rate.
@@ -100,7 +101,7 @@ class HasSimilarContent(Edge):
         "__destination_buffer" : "The name of the buffer of the destination node that the similarity was computed with."
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataEdge.__defining_data__ | {
         "source_buffer",
         "destination_buffer"
     }
@@ -113,33 +114,31 @@ class HasSimilarContent(Edge):
     min_weight = WeightSetting(0.5)
     max_weight = WeightSetting(1.0)
 
-    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True) -> None:
-        super().__init__(source, destination, auto_write=auto_write)
-        self.__source_buffer = "global_buffer"
-        self.__destination_buffer = "global_buffer"
+    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True, source_buffer : Literal["read_buffer", "write_buffer", "global_buffer"], destination_buffer : Literal["read_buffer", "write_buffer", "global_buffer"]) -> None:
+        super().__init__(source, destination, auto_write=auto_write, source_buffer = source_buffer, destination_buffer = destination_buffer)
         
     @property
-    def source_buffer(self) -> str:
+    def source_buffer(self) -> Literal["read_buffer", "write_buffer", "global_buffer"]:
         """
         Returns the name of the content selected for comparison in the source node.
         """
-        return self.__source_buffer
+        return self.__source_buffer # type: ignore I don't where you decided it was just str...
     
     @source_buffer.setter
-    def source_buffer(self, name : str):
+    def source_buffer(self, name : Literal["read_buffer", "write_buffer", "global_buffer"]):
         if name not in ("read_buffer", "write_buffer", "global_buffer"):
             raise ValueError("Diff node buffers can only be set to one of ('read_buffer', 'write_buffer', 'global_buffer')")
         self.__source_buffer = name
     
     @property
-    def destination_buffer(self) -> str:
+    def destination_buffer(self) -> Literal["read_buffer", "write_buffer", "global_buffer"]:
         """
         Returns the name of the content selected for comparison in the destination node.
         """
-        return self.__destination_buffer
+        return self.__destination_buffer # type: ignore
     
     @destination_buffer.setter
-    def destination_buffer(self, name : str):
+    def destination_buffer(self, name : Literal["read_buffer", "write_buffer", "global_buffer"]):
         if name not in ("read_buffer", "write_buffer", "global_buffer"):
             raise ValueError("Diff node buffers can only be set to one of ('read_buffer', 'write_buffer', 'global_buffer')")
         self.__destination_buffer = name
@@ -148,4 +147,4 @@ class HasSimilarContent(Edge):
 
 
 
-del Arrow, Color, WeightSetting, Connection, Data, Diff, Edge, File, Handle, Socket, Vertex, logger
\ No newline at end of file
+del Arrow, DataEdge, Color, WeightSetting, Connection, Data, Diff, Edge, File, Handle, Socket, Vertex, logger
\ No newline at end of file
