@@ -5,7 +5,7 @@ This module contains Edge and Arrow subclasses for this behavioral package.
 from .....logger import logger
 from ...config import SwitchSetting
 from ...colors import Color
-from ...graph import Arrow, Edge, Vertex
+from ...graph import Arrow, DataEdge, Edge, Vertex
 from ..filesystem.entities import Directory, File
 from .entities import Call, Process, Thread
 
@@ -115,17 +115,17 @@ class NextSignificantCall(Arrow):
 
 
 
-class StartedThread(Edge):
+class StartedThread(DataEdge):
 
     """
     This kind of edge indicates that a system call started a new thread.
     """
 
     __slots__ = {
-        "remote" : "Indicates if the started thread was started in another process (remote thread)"
+        "__remote" : "Indicates if the started thread was started in another process (remote thread)"
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataEdge.__defining_data__ | {
         "remote"
     }
 
@@ -134,9 +134,21 @@ class StartedThread(Edge):
     source : Call
     destination : Thread
 
-    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True) -> None:
-        super().__init__(source, destination, auto_write=auto_write)
-        self.remote : bool = False
+    def __init__(self, source: Vertex, destination: Vertex, *, auto_write: bool = True, remote : bool) -> None:
+        super().__init__(source, destination, auto_write = auto_write, remote = remote)
+
+    @property
+    def remote(self) -> bool:
+        """
+        Indicates if this the started Thread belongs to a different Process than the one that started it.
+        """
+        return self.__remote
+    
+    @remote.setter
+    def remote(self, r : bool):
+        if not isinstance(r, bool):
+            raise TypeError(f"Expected bool, got '{type(r).__name__}'")
+        self.__remote = r
     
 
 
@@ -170,4 +182,4 @@ class StartedProcess(Edge):
 
 
 
-del Arrow, Call, Color, SwitchSetting, Directory, Edge, File, Process, Thread, Vertex, logger
\ No newline at end of file
+del Arrow, DataEdge, Call, Color, SwitchSetting, Directory, Edge, File, Process, Thread, Vertex, logger
\ No newline at end of file
