@@ -7,7 +7,7 @@ import pathlib
 from .....logger import logger
 from ...config import ColorSetting, SizeSetting
 from ...colors import Color
-from ...graph import UniqueVertex, Vertex
+from ...graph import DataVertex
 
 __all__ = ["File", "Directory", "Handle"]
 
@@ -17,7 +17,7 @@ __all__ = ["File", "Directory", "Handle"]
 
 logger.info("Loading entities from {} library.".format(__name__.rpartition(".")[0].rpartition(".")[2]))
 
-class File(UniqueVertex):
+class File(DataVertex):
 
     """
     A file vertex. Represents a (real) file opened during execution.
@@ -27,7 +27,7 @@ class File(UniqueVertex):
         "__path" : "The path to the file in the file system",
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataVertex.__defining_data__ | {
         "path"
     }
 
@@ -36,18 +36,14 @@ class File(UniqueVertex):
 
     deleted_file_color = ColorSetting(Color(100, 100, 100))
 
-    def __init__(self, *, parent: Vertex | None = None) -> None:
-        from pathlib import PurePath
-        super().__init__(parent=parent)
-        self.__path : PurePath | None = None
+    def __init__(self, *, path : pathlib.PurePath) -> None:
+        super().__init__(path = path)
 
     @property
     def path(self) -> pathlib.PurePath:
         """
         The absolute path to the file.
         """
-        if self.__path is None:
-            raise RuntimeError("Got a File without path.")
         return self.__path
 
     @path.setter
@@ -84,7 +80,7 @@ class File(UniqueVertex):
 
 
 
-class Directory(UniqueVertex):
+class Directory(DataVertex):
 
     """
     A directory vertex. Represents a (real) directory opened during execution.
@@ -94,27 +90,23 @@ class Directory(UniqueVertex):
         "__path" : "The path to the directory in the file system",
     }
 
-    __pickle_slots__ = {
+    __defining_data__ = DataVertex.__defining_data__ | {
         "path"
-        }
+    }
     
     default_color = ColorSetting(Color(0, 100, 0))
     default_size = SizeSetting(2.5)
 
     deleted_directory_color = ColorSetting(Color(100, 100, 100))
 
-    def __init__(self, *, parent: Vertex | None = None) -> None:
-        from pathlib import PurePath
-        super().__init__(parent=parent)
-        self.__path : PurePath | None = None
+    def __init__(self, *, path : pathlib.PurePath) -> None:
+        super().__init__(path = path)
 
     @property
     def path(self) -> pathlib.PurePath:
         """
         The absolute path to the directory.
         """
-        if self.__path is None:
-            raise RuntimeError("Got a File without path.")
         return self.__path
 
     @path.setter
@@ -146,64 +138,17 @@ class Directory(UniqueVertex):
 
 
 
-class Handle(UniqueVertex):
+class Handle(DataVertex):
 
     """
     A handle vertex. Represents a file handle, used when a program opens a file.
     """
 
-    __slots__ = {
-        # For all objects:
-        "synchronize" : "If the handle can perform synchronization operations on the object",
-
-        # For file objects:
-        "read" : "If the handle has the right to read the file",
-        "write" : "If the handle has the right to write to the file",
-        "append" : "If the handle has the right to append to the file",
-        "execute" : "If the handle has the right to execute the file",
-        "read_attributes" : "If the handle has the right to read file attributes",
-        "write_attributes" : "If the handle can change the file attributes",
-        "read_extended_attributes" : "If the handle can read the extended file attributes",
-        "write_extended_attributes" : "If the handle can change the extended file attributes",
-
-        # For directory objects:
-        "list_directory" : "If the handle has the right to list the items in the given directory",
-        "traverse" : "If the handle has the right to traverse the directory and use it to access subforlders/files"
-    }
-
-    __pickle_slots__ = {
-        "synchronize",
-        "read",
-        "write",
-        "append",
-        "execute",
-        "read_attributes",
-        "write_attributes",
-        "read_extended_attributes",
-        "write_extended_attributes",
-        "list_directory",
-        "traverse"
-    }
-
     default_color = ColorSetting(Color(128, 255, 50))
     default_size = SizeSetting(1.5)
 
-    def __init__(self, *, parent: Vertex | None = None) -> None:
-        super().__init__(parent=parent)
-
-        self.synchronize : bool = False
-
-        self.read : bool = False
-        self.write : bool = False
-        self.append : bool = False
-        self.execute : bool = False
-        self.read_attributes : bool = False
-        self.write_attributes : bool = False
-        self.read_extended_attributes : bool = False
-        self.write_extended_attributes : bool = False
-
-        self.list_directory : bool = False
-        self.traverse : bool = False
+    def __init__(self) -> None:
+        super().__init__()
     
     @property
     def file(self) -> File | Directory:
@@ -220,4 +165,4 @@ class Handle(UniqueVertex):
 
 
 
-del Color, ColorSetting, SizeSetting, UniqueVertex, Vertex, logger, pathlib
\ No newline at end of file
+del Color, ColorSetting, SizeSetting, DataVertex, logger, pathlib
\ No newline at end of file
