@@ -2,6 +2,7 @@
 This module contains integration protocols for this behavioral package.
 """
 
+from pathlib import PurePath
 from typing import Type
 
 from .....logger import logger
@@ -12,6 +13,7 @@ from ...utils import chrono
 from ..execution.entities import Call
 from ..execution.utils import CallHandler
 from . import entities, relations
+from Viper.collections.isomorph import IsoDict
 
 __all__ = ["NewFile", "declare_existing_file"]
 
@@ -45,23 +47,25 @@ for name, cls in filter(lambda c : isinstance(c[1], type) and issubclass(c[1], A
         __file_to_dir_cls_table[cls] = namespace[name[:-4] + "Directory"]
     del name, cls
 
-__active_handles : dict[entities.Handle, str] = {}
+__active_handles : IsoDict[entities.Handle, str] = IsoDict()
 __inverted_handles : dict[str, entities.Handle] = {}
 __existing_files : dict[str, entities.File] = {}
 __all_files : dict[str, list[entities.File]] = {}
 
-def declare_existing_file(path : str):
+def declare_existing_file(path : str | PurePath):
     """
     Declares an existing file in the execution environment.
     """
-    from ...utils import is_path
+    from ...utils import is_path, path_factory
     from .entities import File
+    from pathlib import PurePath
+    if isinstance(path, PurePath):
+        path = str(path)
     if not isinstance(path, str):
         raise TypeError("Expected str, got " + repr(type(path).__name__))
     if not is_path(path):
         raise ValueError("Expected a valid file path for the execution platform, got " + repr(path))
-    f = File()
-    f.path = path
+    f = File(path = path_factory(path))
     if path not in __all_files:
         __all_files[path] = []
     __all_files[path].append(f)
@@ -78,6 +82,7 @@ def integrate_file_creation(c : Call):
     Creates a File/Directory vertex if it does not exists.
     """
     from .....logger import logger
+    from ...utils import path_factory
     from .entities import File, Handle
     from .relations import CreatesFile, HasHandle, UsesFile
     if c.status == 1:
@@ -91,8 +96,7 @@ def integrate_file_creation(c : Call):
             f = __existing_files[c.arguments.filepath]
             existed = True
         else:
-            f = File()
-            f.path = c.arguments.filepath
+            f = File(path = path_factory(c.arguments.filepath))
             existed = False
         CreatesFile(c, h)
         if not f in set(h.neighbors()):
@@ -129,6 +133,7 @@ def integrate_file_opening(c : Call):
     Creates an File/Directory vertex.
     """
     from .....logger import logger
+    from ...utils import path_factory
     from .entities import File, Handle
     from .relations import HasHandle, Opens, UsesFile
     if c.status == 1:
@@ -142,8 +147,7 @@ def integrate_file_opening(c : Call):
             f = __existing_files[c.arguments.filepath]
             existed = True
         else:
-            f = File()
-            f.path = c.arguments.filepath
+            f = File(path = path_factory(c.arguments.filepath))
             existed = False
         Opens(c, h)
         if not f in set(h.neighbors()):
@@ -168,7 +172,8 @@ def integrate_file_closing(c : Call):
     if c.status == 1 and c.arguments.handle in __inverted_handles:
         logger.debug("File closed.")
         h = __inverted_handles.pop(c.arguments.handle)
-        __active_handles.pop(h)
+        if h in __active_handles:
+            __active_handles.pop(h)
         Closes(c, h)
 
 @chrono
@@ -184,9 +189,7 @@ def integrate_file_reading(c : Call):
     if c.status == 1 and c.arguments.file_handle in __inverted_handles:
         logger.debug("Reading from file.")
         h = __inverted_handles[c.arguments.file_handle]
-        d = Data()
-        d.time = c.time
-        d.set_data(c.arguments.buffer)
+        d = Data(time = c.time, data = c.arguments.buffer.encode())
         f = h.file
         Reads(d, c)
         Conveys(h, d)
@@ -206,9 +209,7 @@ def integrate_file_writing(c : Call):
     if c.status == 1 and c.arguments.file_handle in __inverted_handles:
         logger.debug("Writing to file.")
         h = __inverted_handles[c.arguments.file_handle]
-        d = Data()
-        d.time = c.time
-        d.set_data(c.arguments.buffer)
+        d = Data(time = c.time, data = c.arguments.buffer.encode())
         f = h.file
         Writes(c, d)
         Conveys(d, h)
@@ -221,6 +222,7 @@ def integrate_file_copying(c : Call):
     Creates a File vertex, and links it to the original file it was copied from.
     """
     from .....logger import logger
+    from ...utils import path_factory
     from .entities import File
     from .relations import IsCopyiedInto
     if c.status == 1:
@@ -228,8 +230,7 @@ def integrate_file_copying(c : Call):
         if c.arguments.oldfilepath in __existing_files:
             fs = __existing_files[c.arguments.oldfilepath]
         else:
-            fs = File()
-            fs.path = c.arguments.oldfilepath
+            fs = File(path = path_factory(c.arguments.oldfilepath))
             __existing_files[c.arguments.oldfilepath] = fs
             if c.arguments.oldfilepath not in __all_files:
                 __all_files[c.arguments.oldfilepath] = []
@@ -238,8 +239,7 @@ def integrate_file_copying(c : Call):
         if c.arguments.newfilepath in __existing_files:
             fd = __existing_files[c.arguments.newfilepath]
         else:
-            fd = File()
-            fd.path = c.arguments.newfilepath
+            fd = File(path = path_factory(c.arguments.newfilepath))
             __existing_files[c.arguments.newfilepath] = fd
             if c.arguments.newfilepath not in __all_files:
                 __all_files[c.arguments.newfilepath] = []
@@ -251,12 +251,12 @@ def integrate_file_copying(c : Call):
 def integrate_file_deleting(c : Call):
     # You need to make it possible to have multiple times the same file name...
     from .....logger import logger
+    from ...utils import path_factory
     from .entities import File
     if c.status == 1:
         logger.debug("Deleting file.")
         if c.arguments.filepath not in __existing_files:
-            f = File()
-            f.path = c.arguments.filepath
+            f = File(path = path_factory(c.arguments.filepath))
             __existing_files[c.arguments.filepath] = f
             if c.arguments.filepath not in __all_files:
                 __all_files[c.arguments.filepath] = []
@@ -307,7 +307,8 @@ def build_fs_tree(e : BuildingPhase):
     from typing import Type
 
     from .....logger import logger
-    from ...graph import Edge, Graph, find_or_create
+    from ...graph import Edge, Graph
+    from ...utils import active_builder
     from ..execution import Runs
     from ..network import Host
     from .entities import Directory, File
@@ -317,8 +318,7 @@ def build_fs_tree(e : BuildingPhase):
 
 
     def mutate(f : File):
-        d = Directory()
-        d.path = str(f.path)
+        d = Directory(path = f.path)
         d.size = f.size
         graphs = Graph.active_graphs()
         for e in f.edges.copy():
@@ -366,8 +366,7 @@ def build_fs_tree(e : BuildingPhase):
         logger.debug("Creating missing directory nodes.")
         for path, filelist in directories.items():
             if not filelist:
-                d = Directory()
-                d.path = str(path)
+                d = Directory(path = path)
                 filelist.append(d)
             
         logger.debug("Building file system graph.")
@@ -382,7 +381,10 @@ def build_fs_tree(e : BuildingPhase):
                     Contains(parent_node, node)
                     work.add(parent_node)
             else:
-                HasDrive(find_or_create(Host, domain = "host")[0], node)
+                builder = active_builder()
+                if builder is None:
+                    raise RuntimeError("Integration functions called outside of a buidling cycle!")
+                HasDrive(builder.host, node)
         
         # Deleting references to irrelevent (mutated) File nodes
         __existing_files.clear()
@@ -400,4 +402,4 @@ BuildingPhase.add_callback(build_fs_tree)
 
 
 
-del Arrow, BuildingPhase, Call, CallHandler, Edge, Event, Type, build_fs_tree, chrono, entities, integrate_file_closing, integrate_file_copying, integrate_file_creation, integrate_file_deleting, integrate_file_opening, integrate_file_reading, integrate_file_writing, logger, namespace, relations
\ No newline at end of file
+del Arrow, BuildingPhase, Call, CallHandler, Edge, Event, Type, PurePath, IsoDict, build_fs_tree, chrono, entities, integrate_file_closing, integrate_file_copying, integrate_file_creation, integrate_file_deleting, integrate_file_opening, integrate_file_reading, integrate_file_writing, logger, namespace, relations
\ No newline at end of file
