@@ -2,7 +2,7 @@
 This package defines the graph-related subclasses that modelize importation mechanisms.
 """
 
-from ....logger import logger
+from .....logger import logger
 
 __all__ = ["entities", "relations", "integration"]
 
