@@ -17,20 +17,15 @@ logger.info("Loading integrations from {} library.".format(__name__.rpartition("
 
 @chrono
 def link(e : NewFile):
-    from ...graph import find_or_create
     from .entities import Import
     from .relations import IsFile
     if e.file.path.suffix.lower().endswith(".dll"):
-        I, ok = find_or_create(Import, path = e.file.name)
-        if not ok:
-            import re
-            expr1 = re.compile(r"([a-zA-Z0-9_.]+)(\..*?)$")
-            match = expr1.search(e.file.name)
-            if match:
-                I.name = match.group(1).lower()
-            else:
-                I.name = e.file.name
-        IsFile(I, e.file)
+        for i in Import:
+            if e.file.path == i.path:
+                break
+        else:
+            i = Import(path = e.file.path)
+        IsFile(i, e.file)
 
 NewFile.add_callback(link)
 
