@@ -7,7 +7,7 @@ from ...utils import chrono
 from ..execution.entities import Call
 from ..execution.utils import CallHandler
 from . import entities, relations
-
+from Viper.collections.isomorph import IsoDict
 __all__ = []
 
 
@@ -16,9 +16,9 @@ __all__ = []
 
 logger.info("Loading integrations from {} library.".format(__name__.rpartition(".")[0].rpartition(".")[2]))
 
-__active_connections : dict[entities.Socket, entities.Connection] = {}
-__active_handles : dict[entities.Socket, int] = {}
-__inverted_handles : dict[int, entities.Socket] = {}
+__active_connections : IsoDict[entities.Socket, entities.Connection] = IsoDict()
+__active_handles : IsoDict[entities.Socket, int] = IsoDict()
+__inverted_handles : IsoDict[int, entities.Socket] = IsoDict()
 
 @chrono
 def integrate_socket_creation(c : Call):
@@ -30,12 +30,14 @@ def integrate_socket_creation(c : Call):
     from .relations import CreatesSocket, HasSocket
     if c.status == 1:
         logger.debug("Socket created.")
-        s = Socket()
+        s = Socket(
+            handle = c.arguments.socket,
+            family = Socket.families[c.arguments.af][0],
+            protocol = Socket.protocols[c.arguments.protocol][0],
+            type = Socket.types[c.arguments.type][0]
+            )
         __active_handles[s] = c.arguments.socket
         __inverted_handles[c.arguments.socket] = s
-        s.family = Socket.families[c.arguments.af][0]
-        s.protocol = Socket.protocols[c.arguments.protocol][0]
-        s.type = Socket.types[c.arguments.type][0]
         if s in __active_connections:
             __active_connections.pop(s)
         CreatesSocket(c, s)
@@ -47,8 +49,8 @@ def integrate_socket_binding(c : Call):
     Creates a Connection vertex and binds it to a local address.
     """
     from .....logger import logger
-    from ...graph import find_or_create
-    from .entities import Connection, Host
+    from ...utils import active_builder
+    from .entities import Connection
     from .relations import Binds, Communicates, HasConnection
     if c.status == 1:
         logger.debug("Socket bound.")
@@ -60,12 +62,15 @@ def integrate_socket_binding(c : Call):
             return
         __active_connections[s] = l
         HasConnection(s, l)
-        Communicates(l, find_or_create(Host, domain = "host")[0])
-        b = Binds(c, l)
+        builder = active_builder()
+        if builder is None:
+            raise RuntimeError("Integration functions called outside of a buidling cycle!")
+        Communicates(l, builder.host)
         if s.family == "InterNetwork":
-            b.src = (c.arguments.ip_address, c.arguments.port)
+            b = Binds(c, l, src = (c.arguments.ip_address, c.arguments.port))
             l.src = b.src
         else:
+            b = Binds(c, l, src = ("Unknown", -1))
             logger.error("I don't know how to handle this type of address:\n{}".format(s))
 
 @chrono
@@ -74,7 +79,7 @@ def integrate_socket_connection(c : Call):
     Sets the remote address of a connection.
     """
     from .....logger import logger
-    from ...graph import find_or_create
+    from ...utils import active_builder
     from .entities import Connection, Host
     from .relations import Communicates, Connects, HasConnection
     if c.status == 1:
@@ -94,14 +99,22 @@ def integrate_socket_connection(c : Call):
         else:
             l = Connection()
         HasConnection(s, l)
-        Communicates(l, find_or_create(Host, domain = "host")[0])
+        builder = active_builder()
+        if builder is None:
+            raise RuntimeError("Integration functions called outside of a buidling cycle!")
+        Communicates(l, builder.host)
         __active_connections[s] = l
-        b = Connects(c, l)
-        Communicates(l, find_or_create(Host, address = c.arguments.ip_address)[0])
-        if s.family == "InterNetwork":
-            b.dest = (c.arguments.ip_address, c.arguments.port)
-            l.dest = b.dest
+        for dest in Host:
+            if dest.address == c.arguments.ip_address:
+                break
         else:
+            dest = Host(address = c.arguments.ip_address, name = "", domain = "", platform = "")
+        Communicates(l, dest)
+        if s.family == "InterNetwork":
+            b = Connects(c, l, dst = (c.arguments.ip_address, c.arguments.port))
+            l.dst = b.dst
+        else:
+            b = Connects(c, l, dst = ("Unknown", -1))
             logger.error("I don't know how to handle this type of address:\n{}".format(s))
 
 
@@ -127,7 +140,6 @@ def integrate_socket_accepting(c : Call):
     Sets a connection remote address through an accept system call. Also creates a new socket object.
     """
     from .....logger import logger
-    from ...graph import find_or_create
     from .entities import Connection, Host, Socket
     from .relations import Accepts, Communicates, Duplicates, HasConnection
     if c.status == 1:
@@ -137,11 +149,8 @@ def integrate_socket_accepting(c : Call):
         else:
             logger.warning("Accepting from non-existing socket.")
             return
-        s1 = Socket()
+        s1 = Socket(handle = c.return_value, family = s.family, protocol = s.protocol, type = s.type)
         l1 = Connection()
-        s1.family = s.family
-        s1.protocol = s.protocol
-        s1.type = s.type
         __active_handles[s1] = c.return_value
         __inverted_handles[c.return_value] = s1
         if s not in __active_connections:
@@ -149,13 +158,20 @@ def integrate_socket_accepting(c : Call):
         else:
             l = __active_connections[s]
             l1.src = l.src
-        b = Accepts(c, l1)
         HasConnection(s1, l1)
         Duplicates(s, s1)
-        Communicates(l1, find_or_create(Host, address = c.arguments.ip_address)[0])
+        for dest in Host:
+            if dest.address == c.arguments.ip_address:
+                break
+        else:
+            dest = Host(address = c.arguments.ip_address, name = "", domain = "", platform = "")
+        Communicates(l1, dest)
         if s1.family == "InterNetwork":
-            b.dest = (c.arguments.ip_address, c.arguments.port)
-            l1.dest = b.dest
+            b = Accepts(c, l1, dst = (c.arguments.ip_address, c.arguments.port))
+            l1.dst = b.dst
+        else:
+            b = Accepts(c, l1, dst = ("Unknown", -1))
+            logger.error("I don't know how to handle this type of address:\n{}".format(s))
 
 @chrono
 def integrate_socket_close(c : Call):
@@ -220,9 +236,7 @@ def integrate_socket_send(c : Call):
             logger.warning("Sending through unbound socket.")
         else:
             l = __active_connections[s]
-            d = Data()
-            d.time = c.time
-            d.set_data(c.arguments.buffer)
+            d = Data(time = c.time, data = c.arguments.buffer.encode())
             Sends(c, d)
             Conveys(d, l)
             l.volume += len(d.data)
@@ -248,9 +262,7 @@ def integrate_socket_recv(c : Call):
             logger.warning("Receiving through unbound socket.")
         else:
             l = __active_connections[s]
-            d = Data()
-            d.time = c.time
-            d.set_data(c.arguments.buffer)
+            d = Data(time = c.time, data = c.arguments.buffer.encode())
             Receives(d, c)
             Conveys(l, d)
             l.volume += len(d.data)
@@ -291,4 +303,4 @@ CallHandler(integrate_socket_recv, "recv")
 
 
 
-del Call, CallHandler, chrono, entities, integrate_socket_accepting, integrate_socket_binding, integrate_socket_close, integrate_socket_connection, integrate_socket_creation, integrate_socket_listening, integrate_socket_recv, integrate_socket_send, integrate_socket_shutdown, logger, relations
\ No newline at end of file
+del Call, CallHandler, IsoDict, chrono, entities, integrate_socket_accepting, integrate_socket_binding, integrate_socket_close, integrate_socket_connection, integrate_socket_creation, integrate_socket_listening, integrate_socket_recv, integrate_socket_send, integrate_socket_shutdown, logger, relations
\ No newline at end of file
