@@ -8,6 +8,7 @@ from ...utils import chrono
 from ..execution.entities import Call
 from ..execution.utils import CallHandler
 from . import entities, relations
+from Viper.collections.isomorph import IsoDict
 
 __all__ = []
 
@@ -19,35 +20,37 @@ logger.info("Loading integrations from {} library.".format(__name__.rpartition("
 
 __existing_keys : dict[str, entities.Key] = {}
 __inverted_handles : dict[str, entities.Handle] = {}
-__active_handles : dict[entities.Handle, str] = {}
-__existing_entries : dict[entities.Key, dict[str, entities.KeyEntry]] = {}
-__deleted_entries : dict[entities.Key, dict[str, entities.KeyEntry]] = {}
+__active_handles : IsoDict[entities.Handle, str] = IsoDict()
+__existing_entries : dict[str, dict[str, entities.KeyEntry]] = {}
+__deleted_entries : dict[str, dict[str, entities.KeyEntry]] = {}
 
 @chrono
 def __create_key_tree(key : str) -> entities.Key:
     """
     Creates all the missing keys in leading to the final key and returns the leaf Key node.
     """
-    from ...graph import find_or_create
-    from ..network import Host
+    from ...utils import active_builder
     from .entities import Key
     from .relations import HasSubKey
 
     k = None
-    last = find_or_create(Host, domain = "host")[0]
+    builder = active_builder()
+    if builder is None:
+        raise RuntimeError("Integration functions called outside of a buidling cycle!")
+    last = builder.host
     for name in key.replace("\x00", "\uFFFD").split("\\"):
-        name = name.lower().title()
-        k = None
-        for e in last.edges:
-            if isinstance(e, HasSubKey) and e.source is last and e.destination.name == name:
-                k = e.destination
-                break
-        if not k:
-            k = Key()
-            k.name = name
-            HasSubKey(last, k)
-            __existing_keys[k.path.lower()] = k
-        last = k
+        if name:
+            name = name.lower().title()
+            k = None
+            for e in last.edges:
+                if isinstance(e, HasSubKey) and e.source is last and e.destination.name == name:
+                    k = e.destination
+                    break
+            if not k:
+                k = Key(name = name)
+                HasSubKey(last, k)
+                __existing_keys[k.path.lower()] = k
+            last = k
     
     if not k:
         raise RuntimeError("Trying to create an unnamed registry key!")
@@ -73,8 +76,8 @@ def __find_last_key_entry(key : entities.Key, name : str) -> entities.KeyEntry |
     """
     Given a registry key and an entry name, returns the last entry known with this name.
     """
-    if key in __existing_entries and name in __existing_entries[key]:
-        return __existing_entries[key][name]
+    if key.path in __existing_entries and name in __existing_entries[key.path]:
+        return __existing_entries[key.path][name]
 
     
 
@@ -94,7 +97,7 @@ def integrate_key_opening(c : Call):
             k = __existing_keys[c.arguments.regkey.lower()]
         else:
             k = __create_key_tree(c.arguments.regkey)
-        h = Handle()
+        h = Handle(handle = int(c.arguments.key_handle, base=0))
         UsesKey(h, k)
         HasHandle(c.thread.process, h)
         if c.arguments.key_handle.lower() in __inverted_handles:
@@ -154,10 +157,10 @@ def integrate_key_deleting(c : Call):
         k = __create_key_tree(c.arguments.regkey)
         k.color = k.deleted_key_color
         __existing_keys.pop(k.path.lower())
-        if k not in __deleted_entries:
-            __deleted_entries[k] = {}
-        if k in __existing_entries:
-            __deleted_entries[k].update(__existing_entries.pop(k))
+        if k.path not in __deleted_entries:
+            __deleted_entries[k.path] = {}
+        if k.path in __existing_entries:
+            __deleted_entries[k.path].update(__existing_entries.pop(k.path))
 
 @chrono
 def integrate_key_value_querying(c : Call):
@@ -194,13 +197,11 @@ def integrate_key_value_querying(c : Call):
                 logger.warning("There might be an unkown symbolic link in the registry or there is a problem:\nQuerying key '{}' from open key '{}'.".format(sk.path, k.path))
 
         # Creating a new KeyEntry object
-        nv = KeyEntry[c.flags.reg_type]()
-        nv.process_value(c.arguments.value)
-        nv.name = name
+        nv = KeyEntry[c.flags.reg_type](name = name, value = c.arguments.value)
 
-        # Looking for an olod value for this entry
+        # Looking for an old value for this entry
         ov = __find_last_key_entry(sk, nv.name)
-        if not ov or ov.py_type != nv.py_type or ov.value != nv.value or (k in __deleted_entries and ov.name in __deleted_entries[k] and __deleted_entries[k][ov.name] is ov):    # New value is the first or is different : link to previous if there is one.
+        if not ov or ov.py_type != nv.py_type or ov.value != nv.value or (sk.path in __deleted_entries and ov.name in __deleted_entries[sk.path] and __deleted_entries[sk.path][ov.name] is ov):    # New value is the first or is different : link to previous if there is one.
             v = nv
             if ov:
                 ChangesTowards(ov, nv)
@@ -209,10 +210,10 @@ def integrate_key_value_querying(c : Call):
             for g in Graph.active_graphs():
                 g.remove(nv)
         
-        if k not in __existing_entries:
-            __existing_entries[k] = {}
-        __existing_entries[k][v.name] = v
-        if v not in sk.neighbors():
+        if sk.path not in __existing_entries:
+            __existing_entries[sk.path] = {}
+        __existing_entries[sk.path][v.name] = v
+        if not any(v is u for u in sk.entries):
             HasEntry(sk, v)
         QueriesEntry(v, h)
 
@@ -248,13 +249,11 @@ def integrate_key_value_setting(c : Call):
                 logger.warning("There might be an unkown symbolic link in the registry or there is a problem:\nSetting key '{}' from open key '{}'.".format(sk.path, k.path))
 
         # Creating a new KeyEntry object
-        nv = KeyEntry[c.flags.reg_type]()
-        nv.process_value(c.arguments.value)
-        nv.name = name
+        nv = KeyEntry[c.flags.reg_type](name = name, value = c.arguments.value)
 
         # Looking for an olod value for this entry
         ov = __find_last_key_entry(sk, nv.name)
-        if not ov or ov.py_type != nv.py_type or ov.value != nv.value or (k in __deleted_entries and ov.name in __deleted_entries[k] and __deleted_entries[k][ov.name] is ov):    # New value is the first or is different : link to previous if there is one.
+        if not ov or ov.py_type != nv.py_type or ov.value != nv.value or (sk.path in __deleted_entries and ov.name in __deleted_entries[sk.path] and __deleted_entries[sk.path][ov.name] is ov):    # New value is the first or is different : link to previous if there is one.
             v = nv
             if ov:
                 ChangesTowards(ov, nv)
@@ -263,10 +262,10 @@ def integrate_key_value_setting(c : Call):
             for g in Graph.active_graphs():
                 g.remove(nv)
         
-        if k not in __existing_entries:
-            __existing_entries[k] = {}
-        __existing_entries[k][v.name] = v
-        if v not in sk.neighbors():
+        if sk.path not in __existing_entries:
+            __existing_entries[sk.path] = {}
+        __existing_entries[sk.path][v.name] = v
+        if not any(v is u for u in sk.entries):
             HasEntry(sk, v)
         SetsEntry(h, v)
 
@@ -308,21 +307,21 @@ def integrate_key_value_deleting(c : Call):
         if not ov:                                                  # New value is the first.
             raise RuntimeError("Here is what you got " + repr(c))
         else:                                                       # Old value : link to new value and delete new.
-            nv = KeyEntry[ov.reg_type]()                            # type: ignore      Indeed the type-checker is too dump to know about __class_getitem__...
+            nv = KeyEntry[ov.reg_type](name = name, value = ov.value)                            # type: ignore      Indeed the type-checker is too dump to know about __class_getitem__...
             ChangesTowards(ov, nv)
             v = nv
             nv.process_value(ov.value)
             for g in Graph.active_graphs():
                 g.remove(nv)
         
-        if k not in __existing_entries:
-            __existing_entries[k] = {}
-        __existing_entries[k][v.name] = v
-        if k not in __deleted_entries:
-            __deleted_entries[k] = {}
-        __deleted_entries[k][v.name] = v
+        if sk.path not in __existing_entries:
+            __existing_entries[sk.path] = {}
+        __existing_entries[sk.path][v.name] = v
+        if sk.path not in __deleted_entries:
+            __deleted_entries[sk.path] = {}
+        __deleted_entries[sk.path][v.name] = v
         v.color = v.deleted_key_entry_color
-        if v not in sk.neighbors():
+        if not any(v is u for u in sk.entries):
             HasEntry(sk, v)
         DeletesEntry(h, v)
 
@@ -431,4 +430,4 @@ BuildingPhase.add_callback(find_filesystem_references)
 
 
 
-del BuildingPhase, Call, CallHandler, chrono, entities, find_filesystem_references, integrate_key_closing, integrate_key_deleting, integrate_key_enumeration, integrate_key_opening, integrate_key_value_deleting, integrate_key_value_querying, integrate_key_value_setting, logger, relations
\ No newline at end of file
+del BuildingPhase, Call, CallHandler, IsoDict, chrono, entities, find_filesystem_references, integrate_key_closing, integrate_key_deleting, integrate_key_enumeration, integrate_key_opening, integrate_key_value_deleting, integrate_key_value_querying, integrate_key_value_setting, logger, relations
\ No newline at end of file
