@@ -41,7 +41,7 @@ def extract(rack_path : Path, cp_extracted : Path | None = None):
 
         import os
         from pickle import dump, load, UnpicklingError
-        from Viper.better_threading import Future, DaemonThread
+        from Boa.parallel.thread import Future, DaemonThread
         from ..bakery.source.graph import Graph
 
         if rack.perf:
