@@ -1,23 +1,21 @@
 """
 This module defines MetaGraphs, MetaVertices, MetaEdges and MetaArrows. They allow you to create graph patterns.
-Look at these classes.
+Look at these classes!
 """
 
 from types import UnionType
-from typing import Any, Iterable, Iterator, Optional
-
-from baguette.bakery.source.graph import Edge
-from baguette.croutons.source.evaluator import Evaluator
-
+from typing import Any, Iterable, Iterator, Literal, Never, Optional, overload
 from ...bakery.source.colors import Color
-from ...bakery.source.graph import Arrow, Edge, Graph, UniqueVertex, Vertex
-from ...bakery.source.types.execution import *
-from ...bakery.source.types.filesystem import *
-from ...bakery.source.types.network import *
+from ...bakery.source.graph import Arrow, Edge, FrozenGraph, Graph, Vertex, _GraphStateDict
 from .evaluator import Evaluator
 
+__all__ =   ["MetaVertex", "MetaEdge", "MetaArrow", "MetaGraph", "FrozenMetaGraph"]
 
-class MetaVertex(UniqueVertex):
+
+
+
+
+class MetaVertex(Vertex):
 
     """
     This class describes a type vertex. One or multiple Vertex subclasses can be associated to it as well as additional conditions.
@@ -42,23 +40,19 @@ class MetaVertex(UniqueVertex):
     __slots__ = {
         "__class" : "The class that this MetaVertex represents.",
         "__condition" : "An additional condition function. Takes a Vertex as an input and tells whether or not it can be a valid match (does not need to check type)",
-        "__color" : "The Color forcefully set to this MetaVertex."
-    }
-
-    __pickle_slots__ = {
-        "cls",
-        "condition",
-        "color"
+        "__color" : "The Color forcefully set to this MetaVertex.",
+        "__hash" : "The computed hash."
     }
 
     def __init__(self, *, parent: Optional[Vertex] = None) -> None:
         from ...bakery.source.colors import Color
         from .evaluator import Evaluator
-        super().__init__(parent=parent)
         self.__class : tuple[type[Vertex], ...] = (Vertex, )
         self.__condition : Evaluator[Vertex, bool] | None = None
         self.__color : Color | None = None
+        self.__hash : int | None = None
         self.edges : set[MetaEdge]
+        super().__init__()
 
     @property
     def color(self) -> Color:
@@ -93,11 +87,13 @@ class MetaVertex(UniqueVertex):
         """
         Sets the class(es) associated with this vertex.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from types import UnionType
 
         from ...bakery.source.graph import Vertex
         if isinstance(cls, UnionType):
-            args : tuple[type[Vertex]] = cls.__args__
+            args : tuple[type[Vertex]] = cls.__args__ # type: ignore
             for c in args:
                 if not isinstance(c, type) or not issubclass(c, Vertex):
                     raise TypeError("Expected subclass of Vertex or tuple of subclasses, got a " + repr(c))
@@ -114,6 +110,8 @@ class MetaVertex(UniqueVertex):
     
     @cls.deleter
     def cls(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from ...bakery.source.graph import Vertex
         self.__class = (Vertex, )
 
@@ -125,22 +123,26 @@ class MetaVertex(UniqueVertex):
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Evaluator[Vertex, bool] | str):
+    def condition(self, cond : Evaluator[Vertex, bool] | str | None):
         """
         Sets the additional condition function for this MetaVertex.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from .evaluator import Evaluator
         if isinstance(cond, str):
             try:
                 cond = Evaluator(cond)
             except SyntaxError as e:
                 raise e from None
-        if not isinstance(cond, Evaluator):
-            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
+        if not isinstance(cond, Evaluator | None):
+            raise TypeError(f"Expected Evaluator, str or None, got '{type(cond).__name__}'")
         self.__condition = cond
 
     @condition.deleter
     def condition(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         self.__condition = None
     
     def match(self, v : Vertex) -> bool:
@@ -149,13 +151,19 @@ class MetaVertex(UniqueVertex):
         """
         return isinstance(v, self.__class) and (self.__condition(v) if self.__condition else True)
     
+    def __contains__(self, v : Vertex) -> bool:
+        """
+        Implements v in self. Equivalent to self.match(v).
+        """
+        return self.match(v)
+    
     def __get_cls_str(self) -> str:
         """
         Returns a string to display the class of a MetaVertex.
         """
         return " | ".join(c.__name__ for c in self.__class)
     
-    def __call__(self, cond : Evaluator[Vertex, bool] | str) :
+    def __call__(self, cond : Evaluator[Vertex, bool] | str):
         """
         Implements self(cond). Sets the condition and returns self.
         """
@@ -166,13 +174,13 @@ class MetaVertex(UniqueVertex):
             raise e from None
         
     def __getstate__(self):
-        s = super().__getstate__()
-        if not s["condition"]:
-            s.pop("condition")
-        return s
+        return super().__getstate__() | {
+            "cls" : self.cls,
+            "condition" : self.condition
+        }
     
     def __setstate__(self, state : dict[str, Any]):
-        self.__condition = None
+        self.__hash = None
         super().__setstate__(state)
 
     @property
@@ -185,6 +193,20 @@ class MetaVertex(UniqueVertex):
     def __repr__(self) -> str:
         return f"{type(self).__name__}[{self.__get_cls_str()}]" + (f"({repr(self.__condition.code)})" if self.__condition else "")
     
+    def __eq__(self, other : Any) -> bool:
+        """
+        Implements self == other.
+        """
+        return isinstance(other, MetaVertex) and set(self.cls) == set(other.cls) and self.condition == other.condition
+    
+    def __hash__(self) -> int:
+        """
+        Implements hash(self).
+        """
+        if self.__hash is None:
+            self.__hash = hash(hash(frozenset(self.cls)) * hash(self.condition))
+        return self.__hash
+
 
 
 
@@ -210,26 +232,22 @@ class MetaEdge(Edge):
     __slots__ = {
         "__class" : "The class that this MetaEdge represents.",
         "__condition" : "An additional condition function. Takes an Edge as an input and tells whether or not it can be a valid match (does not need to check type)",
-        "__color" : "The Color forcefully set to this MetaEdge."
+        "__color" : "The Color forcefully set to this MetaEdge.",
+        "__hash" : "The computed hash."
     }
 
-    __pickle_slots__ = {
-        "cls",
-        "condition",
-        "color"
-    }
-
-    def __init__(self, source: MetaVertex, destination: MetaVertex, *, auto_write: bool = True) -> None:
+    def __init__(self, source: MetaVertex, destination: MetaVertex) -> None:
         from ...bakery.source.colors import Color
         from .evaluator import Evaluator
         if not isinstance(source, MetaVertex) or not isinstance(destination, MetaVertex):
             raise TypeError("Expected two MetaVertices, got " + repr(type(source).__name__) + " and " + repr(type(destination).__name__))
-        super().__init__(source, destination, auto_write=auto_write)
         self.__class : tuple[type[Edge], ...] = (Edge, )
         self.__condition : Evaluator[Edge, bool] | None = None
         self.__color : Color | None = None
+        self.__hash : int | None = None
         self.source : MetaVertex
         self.destination : MetaVertex
+        super().__init__(source, destination, auto_write=False)
 
     @property
     def color(self) -> Color:
@@ -268,11 +286,13 @@ class MetaEdge(Edge):
         """
         Sets the class(es) associated with this edge.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from types import UnionType
 
         from ...bakery.source.graph import Edge
         if isinstance(cls, UnionType):
-            args : tuple[type[Edge]] = cls.__args__
+            args : tuple[type[Edge]] = cls.__args__ # type: ignore
             for c in args:
                 if not isinstance(c, type) or not issubclass(c, Edge):
                     raise TypeError("Expected subclass of Edge or tuple of subclasses, got a " + repr(c))
@@ -289,6 +309,8 @@ class MetaEdge(Edge):
         
     @cls.deleter
     def cls(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from ...bakery.source.graph import Edge
         self.__class = (Edge, )
         
@@ -300,22 +322,26 @@ class MetaEdge(Edge):
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Evaluator[Edge, bool] | str):
+    def condition(self, cond : Evaluator[Edge, bool] | str | None):
         """
         Sets the additional condition function for this MetaEdge.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from .evaluator import Evaluator
         if isinstance(cond, str):
             try:
                 cond = Evaluator(cond)
             except SyntaxError as e:
                 raise e from None
-        if not isinstance(cond, Evaluator):
-            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
+        if not isinstance(cond, Evaluator | None):
+            raise TypeError(f"Expected Evaluator, str or None, got '{type(cond).__name__}'")
         self.__condition = cond
     
     @condition.deleter
     def condition(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         self.__condition = None
     
     def match(self, e : Edge) -> bool:
@@ -324,6 +350,12 @@ class MetaEdge(Edge):
         """
         return isinstance(e, self.__class) and (self.__condition(e) if self.__condition else True)
     
+    def __contains__(self, e : Edge) -> bool:
+        """
+        Implements e in self. Equivalent to self.match(e).
+        """
+        return self.match(e)
+    
     def __get_cls_str(self) -> str:
         """
         Returns a string to display the class of a MetaEdge.
@@ -331,13 +363,13 @@ class MetaEdge(Edge):
         return " | ".join(c.__name__ for c in self.cls)
     
     def __getstate__(self):
-        s = super().__getstate__()
-        if not s["condition"]:
-            s.pop("condition")
-        return s
+        return super().__getstate__() | {
+            "cls" : self.cls,
+            "condition" : self.condition
+        }
     
     def __setstate__(self, state : dict[str, Any]):
-        self.__condition = None
+        self.__hash = None
         super().__setstate__(state)
 
     @property
@@ -363,6 +395,20 @@ class MetaEdge(Edge):
             return self
         except BaseException as e:
             raise e from None
+    
+    def __eq__(self, other : Any) -> bool:
+        """
+        Implements self == other.
+        """
+        return isinstance(other, MetaEdge) and set((self.source, self.destination)) == set((other.source, other.destination)) and set(self.cls) == set(other.cls) and self.condition == other.condition
+    
+    def __hash__(self) -> int:
+        """
+        Implements hash(self).
+        """
+        if self.__hash is None:
+            self.__hash = hash(hash(frozenset(self.cls)) * hash(self.condition) * (hash(self.source) + hash(self.destination)))
+        return self.__hash
 
 
 
@@ -377,20 +423,17 @@ class MetaArrow(Arrow, MetaEdge):
 
     __slots__ = {
         "__class" : "The class that this MetaArrow represents.",
-        "__condition" : "An additional condition function. Takes an Arrow as an input and tells whether or not it can be a valid match (does not need to check type)"
+        "__condition" : "An additional condition function. Takes an Arrow as an input and tells whether or not it can be a valid match (does not need to check type)",
+        "__hash" : "The computed hash."
     }
 
-    __pickle_slots__ = {
-        "cls",
-        "condition"
-    }
-
-    def __init__(self, source: MetaVertex, destination: MetaVertex, *, auto_write: bool = True) -> None:
+    def __init__(self, source: MetaVertex, destination: MetaVertex) -> None:
         from ...bakery.source.graph import Arrow
         from .evaluator import Evaluator
-        super().__init__(source, destination, auto_write=auto_write)
         self.__class : tuple[type[Arrow], ...] = (Arrow, )
         self.__condition : Evaluator[Arrow, bool] | None = None
+        self.__hash : int | None = None
+        super().__init__(source, destination)
 
     @property
     def cls(self) -> tuple[type[Arrow], ...]:
@@ -404,11 +447,13 @@ class MetaArrow(Arrow, MetaEdge):
         """
         Sets the class(es) associated with this arrow.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from types import UnionType
 
         from ...bakery.source.graph import Arrow
         if isinstance(cls, UnionType):
-            args : tuple[type[Arrow]] = cls.__args__
+            args : tuple[type[Arrow]] = cls.__args__ # type: ignore
             for c in args:
                 if not isinstance(c, type) or not issubclass(c, Arrow):
                     raise TypeError("Expected subclass of Arrow or tuple of subclasses, got a " + repr(c))
@@ -425,6 +470,8 @@ class MetaArrow(Arrow, MetaEdge):
         
     @cls.deleter
     def cls(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from ...bakery.source.graph import Arrow
         self.__class = (Arrow, )
         
@@ -436,29 +483,41 @@ class MetaArrow(Arrow, MetaEdge):
         return self.__condition
     
     @condition.setter
-    def condition(self, cond : Evaluator[Arrow, bool] | str):
+    def condition(self, cond : Evaluator[Arrow, bool] | str | None):
         """
         Sets the additional condition function for this MetaArrow.
         """
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         from .evaluator import Evaluator
         if isinstance(cond, str):
             try:
                 cond = Evaluator(cond)
             except SyntaxError as e:
                 raise e from None
-        if not isinstance(cond, Evaluator):
-            raise TypeError(f"Expected Evaluator or str, got '{type(cond).__name__}'")
+        if not isinstance(cond, Evaluator | None):
+            raise TypeError(f"Expected Evaluator, str or None, got '{type(cond).__name__}'")
         self.__condition = cond
     
     @condition.deleter
     def condition(self):
+        if self.__hash is not None:
+            raise RuntimeError(f"__hash__ has been computed: you cannot edit this {type(self).__name__} without breaking container structures")
         self.__condition = None
     
     def __setstate__(self, state : dict[str, Any]):
-        self.__condition = None
+        self.__hash = None
         super().__setstate__(state)
     
+    @overload
+    def match(self, a : Edge) -> Literal[False]:
+        ...
+
+    @overload
     def match(self, a : Arrow) -> bool:
+        ...
+
+    def match(self, a):
         """
         Returns True if the Arrow a has a matching type.
         """
@@ -489,60 +548,118 @@ class MetaArrow(Arrow, MetaEdge):
             return self
         except BaseException as e:
             raise e from None
+        
+    def __eq__(self, other : Any) -> bool:
+        """
+        Implements self == other.
+        """
+        return isinstance(other, MetaArrow) and (self.source, self.destination) == (other.source, other.destination) and set(self.cls) == set(other.cls) and self.condition == other.condition
+    
+    def __hash__(self) -> int:
+        """
+        Implements hash(self).
+        """
+        if self.__hash is None:
+            self.__hash = hash(hash(frozenset(self.cls)) * hash(self.condition) * (hash(self.destination) - hash(self.source)))
+        return self.__hash
+
     
 
 
 
+class _MetaGraphStateDict(_GraphStateDict):
+
+    named_objects : dict[str, MetaEdge | MetaVertex]
 
 class MetaGraph(Graph):
 
     """
     This particular type of graph can only contain MetaVertices, MetaEdges or MetaArrows. It can be used for normal graph exploration.
+
     Note that contrary to normal Graphs, MetaGraphs can hold named vertices and edges:
     >>> MG = MetaGraph()
     >>> MG.File = MetaVertex[File]
     >>> len(MG.vertices)
     1
-    >>> mG.File
+    >>> MG.File
     MetaVertex[File]
+
+    MetaGraphs can also be created by giving a Graph or an iterable of Vertices and Edges to the constructor, in which case, it will make a MetaGraph that matches the structure of the given Graph.
+    It can also receive a MetaGraph or an iterable of MetaVertices and MetaEdges, to perform a MetaGraph copy.
     """
 
+    from itertools import chain as __chain
+    from Viper.collections import IsoSet as __IsoSet, IsoDict as __IsoDict
+    from ...bakery.source.graph import Graph as __Graph
+
     __slots__ = {
         "__named_objects" : "A dict holding named vertices and edges."
     }
 
-    vertices : set[MetaVertex]
-    edges : set[MetaEdge | MetaArrow]
+    vertices : __IsoSet[MetaVertex]
+    edges : __IsoSet[MetaEdge | MetaArrow]
 
-    def __init__(self, g : Graph = Graph()) -> None:
-        if not isinstance(g, Graph):
-            raise TypeError("Exepcted Graph, got " + repr(type(g).__name__))
-        super().__init__()
-        self.__named_objects : dict[str, MetaEdge | MetaVertex] = {}
-        vertex_translation_table : dict[Vertex, MetaVertex] = {}
-        edge_translation_table : dict[Edge, MetaEdge] = {}
-        for v, e, u in g.pairs():
-            if v in vertex_translation_table:
-                Mv = vertex_translation_table[v]
+    def __init__(self, g : Iterable[Vertex | Edge] | Graph = Graph()) -> None:
+        if isinstance(g, Graph):
+            g = MetaGraph.__chain(g.vertices, g.edges)
+        MetaGraph.__Graph.__init__(self)
+        self.__named_objects : "dict[str, MetaEdge | MetaVertex]" = {}
+        vertices : "MetaGraph.__IsoSet[Vertex]" = MetaGraph.__IsoSet()
+        edges : "MetaGraph.__IsoSet[Edge]" = MetaGraph.__IsoSet()
+        all_meta = True
+        for k in g:
+            if isinstance(k, Vertex):
+                vertices.add(k)
+                if not isinstance(k, MetaVertex):
+                    all_meta = False
+            elif isinstance(k, Edge):
+                edges.add(k)
+                if not isinstance(k, MetaEdge):
+                    all_meta = False
             else:
+                raise TypeError(f"Expected iterable of Vertices and Edges, got a '{type(k).__name__}'")
+            
+        if all_meta:
+            for value in MetaGraph.__chain(vertices, edges):
+                if isinstance(value, MetaVertex):
+                    self.vertices.add(value)
+                elif isinstance(value, MetaEdge):
+                    value.write()
+                    self.edges.add(value)
+        else:
+
+            vertex_translation_table : dict[int, MetaVertex] = {}
+
+            for v in vertices:
                 Mv = MetaVertex()
                 Mv.cls = type(v)
-                vertex_translation_table[v] = Mv
+                vertex_translation_table[id(v)] = Mv
                 self.append(Mv)
-            if u in vertex_translation_table:
-                Mu = vertex_translation_table[u]
-            else:
-                Mu = MetaVertex()
-                Mu.cls = type(u)
-                vertex_translation_table[u] = Mu
-                self.append(Mu)
-            if e in edge_translation_table:
-                Me = edge_translation_table[e]
-            else:
-                Me = MetaEdge(Mv, Mu)
-                Me.cls = type(e)
-                edge_translation_table[e] = Me
-                self.append(Me)
+            
+            for e in edges:
+                u, v = e.source, e.destination
+
+                if id(u) not in vertex_translation_table:
+                    Mu = MetaVertex()
+                    Mu.cls = type(u)
+                    vertex_translation_table[id(u)] = Mu
+                    self.append(Mu)
+                
+                if id(v) not in vertex_translation_table:
+                    Mv = MetaVertex()
+                    Mv.cls = type(v)
+                    vertex_translation_table[id(v)] = Mv
+                    self.append(Mv)
+                
+                if isinstance(e, Arrow):
+                    Me = MetaArrow(vertex_translation_table[id(u)], vertex_translation_table[id(v)])
+                    Me.cls = type(e)
+                    self.append(Me)
+                
+                else:
+                    Me = MetaEdge(vertex_translation_table[id(u)], vertex_translation_table[id(v)])
+                    Me.cls = type(e)
+                    self.append(Me)
     
     @property
     def names(self) -> list[str]:
@@ -551,11 +668,6 @@ class MetaGraph(Graph):
         """
         return list(self.__named_objects)
 
-    def add_vertex(self, v: MetaVertex, explore: bool = True) -> None:
-        if not isinstance(v, MetaVertex):
-            raise TypeError("Expected MetaVertex, got " + repr(type(v).__name__))
-        return super().add_vertex(v, explore)
-
     def pairs(self) -> Iterator[tuple[MetaVertex, MetaEdge, MetaVertex]]:
         for u, e, v in super().pairs():
             if not isinstance(u, MetaVertex) or not isinstance(e, MetaEdge) or not isinstance(v, MetaVertex):
@@ -565,10 +677,13 @@ class MetaGraph(Graph):
     def append(self, value: MetaVertex | MetaEdge, explore: bool = False):
         if not isinstance(value, MetaVertex | MetaEdge | MetaArrow):
             raise TypeError("Expected MetaVertex, MetaEdge or MetaArrow, got " + repr(type(value).__name__))
+        if isinstance(value, MetaEdge):
+            value.write()
         return super().append(value, explore)
     
     def extend(self, values: Iterable[MetaVertex | MetaEdge], explore: bool = False):
         from typing import Iterable
+        from ...bakery.source.graph import Graph
         if not isinstance(explore, bool):
             raise TypeError("Expected bool for explore, got " + repr(explore.__class__.__name__))
         if not isinstance(values, Iterable):
@@ -577,8 +692,10 @@ class MetaGraph(Graph):
             for v in values:
                 if not isinstance(v, MetaVertex | MetaEdge | MetaArrow):
                     raise TypeError("Expected iterable of MetaVertex, MetaEdge or MetaArrow, got " + repr(type(v).__name__))
+                if isinstance(v, MetaEdge):
+                    v.write()
                 yield v
-        return super().extend(__checked(), explore)
+        return Graph.extend(self, __checked(), explore)
 
     def __dir__(self) -> list[str]:
         return list(super().__dir__()) + self.names
@@ -591,19 +708,18 @@ class MetaGraph(Graph):
                 raise e from None
             return self.__named_objects[name]
     
-    def __getstate__(self) -> dict[str, Any]:
-        d = super().__getstate__()
-        d["__named_objects"] = self.__named_objects
+    def __getstate__(self) -> _MetaGraphStateDict:
+        d = super().__getstate__() | {"named_objects" : self.__named_objects}
         return d
     
-    def __setstate__(self, state: dict[str, Any]):
-        names = state.pop("__named_objects")
-        self.__named_objects = names
-        return super().__setstate__(state)
+    def __setstate__(self, state: _MetaGraphStateDict):
+        self.__named_objects = state.pop("named_objects")
+        super().__setstate__(state)
     
     def __setattr__(self, name: str, value: Any) -> None:
         try:
             super().__getattribute__(name)
+            return super().__setattr__(name, value)
         except AttributeError:
             if isinstance(value, MetaEdge | MetaVertex):
                 self.__named_objects[name] = value
@@ -625,98 +741,90 @@ class MetaGraph(Graph):
                 self.__named_objects.pop(name)
         super().remove(value)
     
-    def __neighborhood_mapper(self, g : Graph) -> dict[Vertex, set[MetaVertex]]:
+    def __neighborhood_mapper(self, g : Graph) -> __IsoDict[Vertex, __IsoSet[MetaVertex]]:
         """
         Searches the given graph for matches of the metagraph.
         """
 
-        mapping : dict[Vertex, set[MetaVertex]] = {}
+        mapping : MetaGraph.__IsoDict[Vertex, MetaGraph.__IsoSet[MetaVertex]] = MetaGraph.__IsoDict()
 
         for v in g.vertices:
             for Mv in self.vertices:
                 if Mv.match(v):
                     if v not in mapping:
-                        mapping[v] = set()
+                        mapping[v] = MetaGraph.__IsoSet()
                     mapping[v].add(Mv)
         
         return mapping
     
-    def __clear_mapper_deadends(self, mapping : dict[Vertex, set[MetaVertex]]):
+    def __clear_mapper_deadends(self, mapping : __IsoDict[Vertex, __IsoSet[MetaVertex]]):
         """
         Clears the deadends in a mapping returned by self.__neighborhood_mapper():
         (Operates on the mapping itself)
         """
 
-        work = mapping.copy()
+        work = MetaGraph.__IsoDict((v, sMv.copy()) for v, sMv in mapping.items())
 
         while work:
 
             v, sMv = work.popitem()
+            Mv = sMv.pop()
+            if sMv:
+                work[v] = sMv
 
-            to_check_again : set[tuple[Vertex, MetaVertex]] = set()
+            ok = False
+            edge_possibilities : MetaGraph.__IsoDict[MetaEdge, MetaGraph.__IsoSet[Edge]] = MetaGraph.__IsoDict()     # For all MetaNeighbors. If any of them has an empty 
 
-            for Mv in sMv.copy():
+            for Me in Mv.edges:
+
+                edge_possibilities[Me] = MetaGraph.__IsoSet()
+
+                for e in v.edges:
+
+                    if Me.match(e) and (not isinstance(Me, MetaArrow) or (e.source is v) == (Me.source is Mv)):     # MetaEdge or well-oriented MetaArrow
+                        edge_possibilities[Me].add(e)
+                                
+            all_ok = True
+            decisive_neighbors : MetaGraph.__IsoSet[tuple[Vertex, MetaVertex]] = MetaGraph.__IsoSet()
+
+            for Me in edge_possibilities:
 
                 ok = False
-                edge_possibilities : dict[MetaEdge, set[Edge]] = {}     # For all MetaNeighbors. If any of them has an empty 
 
-                for Me in Mv.edges:
+                for e in edge_possibilities[Me]:
 
-                    edge_possibilities[Me] = set()
-
-                    for e in v.edges:
-
-                        if Me.match(e) and (not isinstance(Me, MetaArrow) or (e.source is v) == (Me.source is Mv)):     # MetaEdge or well-oriented MetaArrow
-                            edge_possibilities[Me].add(e)
-                                    
-                all_ok = True
-                decisive_neighbors : set[tuple[Vertex, MetaVertex]] = set()
-
-                for Me in edge_possibilities:
-
-                    ok = False
-
-                    for e in edge_possibilities[Me]:
-
-                        u = e.source if e.source is not v else e.destination
-                        Mu = Me.source if Me.source is not Mv else Me.destination
-                        decisive_neighbors.add((u, Mu))
-                        
-                        if u in mapping and Mu in mapping[u]:
-                            ok = True
-                
-                    if not ok:
-                        all_ok = False
-                
-                if not all_ok:
-                    mapping[v].discard(Mv)
-                    to_check_again.update(decisive_neighbors)
-            
-            new_work : dict[Vertex, set[MetaVertex]] = {}
-
-            for v, Mv in to_check_again:
-                sMv = work.pop(v, set())
-                if v not in new_work:
-                    new_work[v] = sMv
-                new_work[v].add(Mv)
-
-            work.update({v : sMv.intersection(mapping[v] if v in mapping else set()) for v, sMv in new_work.items()})
+                    u = e.source if e.source is not v else e.destination
+                    Mu = Me.source if Me.source is not Mv else Me.destination
+                    decisive_neighbors.add((u, Mu))
                     
-        # for v, sMv in list(mapping.items()):
-        #     if not sMv:
-        #         mapping.pop(v)
+                    if u in mapping and Mu in mapping[u]:
+                        ok = True
             
-    def __expand_subgraph(self, g : Graph, mapping : dict[Vertex, set[MetaVertex]], sub : dict[MetaVertex | MetaEdge, Vertex | Edge], Mv : MetaVertex) -> Iterator[dict[MetaVertex | MetaEdge, Vertex | Edge]]:
+                if not ok:
+                    all_ok = False
+            
+            if not all_ok:
+                mapping[v].discard(Mv)
+                if not mapping[v]:
+                    mapping.pop(v)
+                
+                for u, Mu in decisive_neighbors:
+                    if u in mapping and Mu in mapping[u] and (u is not v or Mu is not Mv):      # Could cause issues in self-connected vertices
+                        if u not in work:
+                            work[u] = MetaGraph.__IsoSet()
+                        work[u].add(Mu)
+            
+    def __expand_subgraph(self, g : Graph, mapping : __IsoDict[Vertex, __IsoSet[MetaVertex]], sub : __IsoDict[MetaVertex | MetaEdge, Vertex | Edge], Mv : MetaVertex) -> Iterator[__IsoDict[MetaVertex | MetaEdge, Vertex | Edge]]:
         """
         Yields all the next steps of the subgraph sub of Graph g in construction by adding all possible vertices that can fit the role of Mv.
         """
         from itertools import product
 
-        existing : dict[Vertex, MetaVertex] = {sub[Mu] : Mu for Mu in Mv.neighbors() if Mu in sub}      # The neighbors of Mv that have already been chosen
+        existing : MetaGraph.__IsoDict[Vertex, MetaVertex] = MetaGraph.__IsoDict((sub[Mu], Mu) for Mu in Mv.neighbors() if Mu in sub)      # The neighbors of Mv that have already been chosen
 
         if not existing:        # We are starting from zero or we reached a new component
             
-            used_Mvs = set(v for v in sub.values() if v in mapping and Mv in mapping[v])        # The candidates for Mv that are already part of the subgraph
+            used_Mvs = MetaGraph.__IsoSet(v for v in sub.values() if v in mapping and Mv in mapping[v])        # The candidates for Mv that are already part of the subgraph
 
             for v in mapping:
                 if Mv in mapping[v] and v not in used_Mvs:
@@ -730,7 +838,7 @@ class MetaGraph(Graph):
 
         u = next(existing_iter)
 
-        vertex_possibilities : set[Vertex] = {v for v in u.neighbors() if v in mapping and Mv in mapping[v]}
+        vertex_possibilities : MetaGraph.__IsoSet[Vertex] = MetaGraph.__IsoSet(v for v in u.neighbors() if v in mapping and Mv in mapping[v])
 
         for u in existing_iter:
             vertex_possibilities.intersection_update({v for v in u.neighbors() if v in mapping and Mv in mapping[v]})
@@ -738,21 +846,21 @@ class MetaGraph(Graph):
         vertex_possibilities.difference_update(sub.values())
 
         for v in vertex_possibilities:
-            edge_possibilities : dict[MetaEdge, set[Edge]] = {}
+            edge_possibilities : MetaGraph.__IsoDict[MetaEdge, MetaGraph.__IsoSet[Edge]] = MetaGraph.__IsoDict()
 
             for u, Mu in existing.items():
 
                 edges = u.edges & v.edges
                 Medges = Mu.edges & Mv.edges
                 for Me in Medges:
-                    edge_possibilities[Me] = set()
+                    edge_possibilities[Me] = MetaGraph.__IsoSet()
                     for e in edges:
                         if Me.match(e) and (not isinstance(Me, MetaArrow) or (e.source is v) == (Me.source is Mv)):
                             edge_possibilities[Me].add(e)
             
             Me_list = list(edge_possibilities)
             for e_list in product(*[edge_possibilities[Me] for Me in Me_list]):
-                if len(set(e_list)) == len(e_list):     # No edge was used as two different MetaEdges between v and one of its neighbors
+                if len(MetaGraph.__IsoSet(e_list)) == len(e_list):     # No edge was used as two different MetaEdges between v and one of its neighbors
                     subv = sub.copy()
                     subv[Mv] = v
                     for Me, e in zip(Me_list, e_list):
@@ -764,9 +872,9 @@ class MetaGraph(Graph):
         Yields successive MetaVertices by exploring the MetaGraph. The next MetaVertex yielded is either a neighbor of one of the previously yielded ones or is the first of a new connected component.
         """
         
-        to_do : set[MetaVertex] = self.vertices.copy()
-        done : set[MetaVertex] = set()
-        component_explorable : set[MetaVertex] = set()
+        to_do : MetaGraph.__IsoSet[MetaVertex] = MetaGraph.__IsoSet(self.vertices)
+        done : MetaGraph.__IsoSet[MetaVertex] = MetaGraph.__IsoSet()
+        component_explorable : MetaGraph.__IsoSet[MetaVertex] = MetaGraph.__IsoSet()
 
         while to_do:
             if not component_explorable:
@@ -776,7 +884,7 @@ class MetaGraph(Graph):
             else:
                 component_done = True
                 for Mv in component_explorable.copy():
-                    sMu = set(Mv.neighbors()) - done
+                    sMu = MetaGraph.__IsoSet(Mv.neighbors()) - done
                     if sMu:
                         component_done = False
                         Mu = sMu.pop()
@@ -811,7 +919,7 @@ class MetaGraph(Graph):
 
         # print("Got {} vertices with openings.".format(len([v for v, sMv in mapping.items() if sMv])))
 
-        def build_iter(g : Graph, mapping : dict[Vertex, set[MetaVertex]], sub : dict[MetaVertex | MetaEdge, Vertex | Edge], i : int) -> Iterator[Graph]:
+        def build_iter(g : Graph, mapping : MetaGraph.__IsoDict[Vertex, MetaGraph.__IsoSet[MetaVertex]], sub : MetaGraph.__IsoDict[MetaVertex | MetaEdge, Vertex | Edge], i : int) -> Iterator[Graph]:
             if i == len(order) - 1:     # Last MetaVertex to append
                 for subi in self.__expand_subgraph(g, mapping, sub, order[i]):
                     gi = Graph()
@@ -821,7 +929,7 @@ class MetaGraph(Graph):
                 for subi in self.__expand_subgraph(g, mapping, sub, order[i]):
                     yield from build_iter(g, mapping, subi, i + 1)
         
-        yield from build_iter(g, mapping, {}, 0)
+        yield from build_iter(g, mapping, MetaGraph.__IsoDict(), 0)
 
         # for i, Mv in enumerate(self.__discover()):
         #     for sub in subgraphs:
@@ -834,10 +942,48 @@ class MetaGraph(Graph):
         #     g.extend(dg.values())
         #     yield g
 
+    def __contains__(self, g : Graph) -> bool:
+        """
+        Implements g in self. Returns True if Graph g matches entirely MetaGraph self.
+        """
+        return g in self.search_iter(g)
 
 
 
 
+
+class FrozenMetaGraph(MetaGraph, FrozenGraph):
+
+    """
+    Frozen (immutable) version of MetaGraphs. Built from a Graph or MetaGraph given to its constructor.
+    """
+
+    from Viper.collections import FrozenIsoSet as __FrozenIsoSet
+    from ...bakery.source.graph import FrozenGraph as __FrozenGraph
+
+    def __init__(self, g: Iterable[Vertex | Edge] | Graph = Graph()) -> None:
+        MetaGraph.__init__(self, g)
+        self.vertices : "FrozenMetaGraph.__FrozenIsoSet[Vertex]" = FrozenMetaGraph.__FrozenIsoSet(self.vertices)
+        self.edges : "FrozenMetaGraph.__FrozenIsoSet[Edge]" = FrozenMetaGraph.__FrozenIsoSet(self.edges)
+
+    def append(self, value: MetaVertex | MetaEdge, explore: bool = False) -> Never:
+        raise AttributeError(f"Cannot append to a '{type(self).__name__}'")
+    
+    def remove(self, value: MetaVertex | MetaEdge) -> Never:
+        raise AttributeError(f"Cannot remove from a '{type(self).__name__}'")
+    
+    def extend(self, values: Iterable[MetaVertex | MetaEdge], explore: bool = False) -> Never:
+        raise AttributeError(f"Cannot extend a '{type(self).__name__}'")
+    
+    def __getstate__(self) -> _MetaGraphStateDict:
+        return MetaGraph.__getstate__(self) | FrozenMetaGraph.__FrozenGraph.__getstate__(self)
+    
+    def __setstate__(self, state: _MetaGraphStateDict):
+        MetaGraph.__setstate__(self, state)
+        FrozenMetaGraph.__FrozenGraph.__setstate__(self, state)
+
+    
+
 # N = 0
 
 # class Square(UniqueVertex):
