@@ -156,13 +156,15 @@ def main():
             return Color(r, g, b)
 
     parser.add_argument("inputs", type=PathSorter(inputs, "input"), default=None, nargs="*" if ("BAGUETTE_INPUTS" in environ or "BAGUETTE_OUTPUTS" in environ) else "+", help="Baguette folders. These should contain the index file resulting from the baking. Can also be folders baguette folders. Defaults to environment variable 'BAGUETTE_INPUTS' (or 'BAGUETTE_OUTPUTS') if set.")
-    parser.add_argument("--extracted", type=PathSorter(extracted, "extracted"), default=None, action="extend", nargs="*", help="the path(s) to the output (copy) extracted match files (.pyt). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_EXTRACTED' or '-' if not set.")
-    parser.add_argument("--pool", type=pool_size, default=pool_size("0.5"), help="the size of the process pool to use to bake in parallel.")
-    parser.add_argument("--maxtime", type=time, default=time("inf"), help="the maximum amount of time spent baking a single baguette. No maxtime by default.")
-    parser.add_argument("-p", "--pattern", type=str, action="extend", choices=["-"] + entries(), nargs="+", help="The metapath names (as in the source.metalib module) to search for. Leave empty or use '-' to search for all defined metagraphs.")
+    
+    parser.add_argument("--extracted", "-e", type=PathSorter(extracted, "extracted"), default=None, action="extend", nargs="*", help="the path(s) to the output (copy) extracted match files (.pyt). Use '-' to leave it to the automatic destination. Defaults to environment variable 'BAGUETTE_EXTRACTED' or '-' if not set.")
+    parser.add_argument("--pool", "-p", type=pool_size, default=pool_size("0.5"), help="the size of the process pool to use to bake in parallel.")
+    parser.add_argument("--maxtime", "-m", type=time, default=time("inf"), help="the maximum amount of time spent baking a single baguette. No maxtime by default.")
+    parser.add_argument("--pattern", "-P", type=str, action="extend", choices=["-"] + entries(), nargs="+", help="The metapath names (as in the source.metalib module) to search for. Leave empty or use '-' to search for all defined metagraphs.")
+    parser.add_argument("--verbosity", "-v", action="count", default=0, help="increases the verbosity of the output.")
+    
     parser.add_argument("--perf", action="store_true", default=False, help="if this is enabled, a performance report will be printed at the end of the baking process.")
     parser.add_argument("--paint", type=paint_color, nargs="+", default=[], help="If a color is given for painting, the matches found will be painted in the visual.gexf file. Must be a valid color name or RGB values.")
-    parser.add_argument("-v", "--verbosity", action="count", default=0, help="increases the verbosity of the output.")
 
     args = parser.parse_args()
 
