@@ -47,7 +47,8 @@ class BaguetteRack:
         "__paint_color" : "The color to use to paint MetaGraph matches in the visual file",
         "__suppressed" : "Indicates if an exception should suppress the index file output",
         "__background_color" : "The color of the background for the visual graph",
-        "__save_filtered_baguette" : "Indicates if the Graph has been saved after applying filters"
+        "__save_filtered_baguette" : "Indicates if the Graph has been saved after applying filters",
+        "__report_type" : "The type of execution report used as a source"
     }
 
     __pickle_slots__ = {
@@ -64,7 +65,8 @@ class BaguetteRack:
         "paint_color",
         "suppressed",
         "background_color",
-        "save_filtered_baguette"
+        "save_filtered_baguette",
+        "report_type"
     }
 
     names = {
@@ -81,6 +83,7 @@ class BaguetteRack:
         from traceback import TracebackException
         from typing import Literal
         from .bakery.source.colors import Color
+        from .bakery.source.parsers import AbstractParser
         self.__working_directory : Path = Path(working_directory)
         self.__perf : bool = False
         self.__filters : list[str] = []
@@ -96,6 +99,7 @@ class BaguetteRack:
         self.__suppressed : bool = False
         self.__background_color : Color = Color.black
         self.__save_filtered_baguette : bool = False
+        self.__report_type : str = AbstractParser.report_name
         if not isinstance(working_directory, str | Path):
             raise TypeError("Expected Path, got " + repr(type(working_directory).__name__))
     
@@ -428,6 +432,27 @@ class BaguetteRack:
         if not isinstance(value, bool):
             raise TypeError("Expected bool, got " + repr(type(value).__name__))
         self.__save_filtered_baguette = value
+
+    @property
+    def report_type(self) -> str:
+        """
+        The type of execution report that this BAGUETTE was compiled from.
+        """
+        return self.__report_type
+    
+    @report_type.setter
+    def report_type(self, value : str):
+        from .bakery.source.parsers import parsers, AbstractParser
+        if not isinstance(value, str):
+            raise TypeError(f"Expected str, got '{type(value).__name__}'")
+        if value not in {p.report_name for p in parsers} | {AbstractParser.report_name}:
+            raise ValueError(f"Expected one of {(AbstractParser.report_name, ) + tuple(p.report_name for p in parsers)}, got '{value}'")
+        self.__report_type = value
+    
+    @report_type.deleter
+    def report_type(self):
+        from .bakery.source.parsers import AbstractParser
+        self.__report_type = AbstractParser.report_name
         
     def export(self):
         """
@@ -480,6 +505,26 @@ class BaguetteRack:
         if not list(self.working_directory.iterdir()):
             rmdir(self.working_directory)
 
+    @property
+    def compilation_failed(self) -> bool:
+        """
+        This value indicates if the compilation resulted in an Exception.
+        """
+        return not self.baked and self.exception != None
+    
+    @property
+    def compilation_interrupted(self) -> bool:
+        """
+        This value indicates if the compilation failed because it took too long or was interrupted.
+        """
+        return self.compilation_failed and self.exception != None and issubclass(self.exception.exc_type, (TimeoutExit, KeyboardInterrupt))
+
+    def exists(self) -> bool:
+        """
+        This method tests whether or not an index file already exist where this one should be exported to.
+        """
+        return self.index.exists()
+
     def load(self) -> Graph:
         """
         Loads and returns the baguette associated to this graph.
